

 
////////////////////////////////////////////////////////////////////////
//
//
//
////////////////////////////////////////////////////////////////////////



package si.game
{

	include "../siImportFile.as"

	
	
	public class Game extends Sprite
	{

		//note: if you turn on debug for the coll system
		//	the redraw region could become much larger
		public static const DEBUG_COLL_SYSTEM:Boolean   = false;	
		public static const DEBUG_STATES:Boolean   	= false;	
		public static const DEBUG_FRAMERATE:Boolean   	= false;	
		public static const DEBUG_MEM:Boolean   	= false;	
				

		//animation timer via flash
		private var game_timer:Timer;

		protected var game_stage:Stage;
		private var screen_size:Point;
		
		private var coll_system:CollSystem = null;

		protected var game_state:int = -1;
		protected var game_state_obj:GameState = null;
		
		private var time_diff:int = 0;
		private var last_time:int;

		
		
		///////////////////////////////////////////////////////////////////////////
		//
		//
		///////////////////////////////////////////////////////////////////////////
		public function Game(in_stage:Stage, in_screen_size:Point, game_state:int, hide_mouse:Boolean):void
		{
			if (hide_mouse)
				Mouse.hide();


			this.game_state = game_state;

			//initialize systems/objs that would be generic to all games
			initTimer();	


			game_stage = in_stage;
			screen_size = in_screen_size;

			//set the frame rate that the swf will interval at
			game_stage.frameRate = 60;


			//add a clipping mask (to clip everything beynd the borders)
			var myMask:Sprite = new Sprite();
			myMask.graphics.beginFill(0x000000);
			myMask.graphics.drawRect(0, 0, screen_size.x, screen_size.y);
			mask = myMask;

		}

		///////////////////////////////////////////////////////////////////////////
		//
		//
		///////////////////////////////////////////////////////////////////////////
		public function initCollSystem():void
		{
			coll_system = new CollSystem(this,new Point(0,400.0));
		}




		///////////////////////////////////////////////////////////////////////////
		//
		//
		///////////////////////////////////////////////////////////////////////////
		public function shutdownCollSystem():void
		{
			if (coll_system != null)
			{
				coll_system.shutdown();
				coll_system = null;
			}				
		}




		///////////////////////////////////////////////////////////////////////////
		//
		//
		///////////////////////////////////////////////////////////////////////////
		public function shutdown():void
		{

			screen_size = null;			
			game_state = -1;
			
			shutdownTimer();
			
			Mouse.show();

		}


		///////////////////////////////////////////////////////////////////////////
		//
		//
		///////////////////////////////////////////////////////////////////////////
		public function initTimer():void
		{
			//create main game timer to run at 60fps
			game_timer = new Timer(16);
//			game_timer = new Timer(1000/90);
			game_timer.start();
			
			//start the total game update timer
			//this controls the game states
			game_timer.addEventListener (TimerEvent.TIMER, update);
			
			
			//set last_time to 0.
			last_time = 0;


		}

		///////////////////////////////////////////////////////////////////////////
		//
		//
		///////////////////////////////////////////////////////////////////////////
		public function shutdownTimer():void
		{
			game_timer.removeEventListener(TimerEvent.TIMER,update);

			game_timer.stop();
			game_timer = null;
		}


	


		///////////////////////////////////////////////////////////////////////////
		//
		//
		///////////////////////////////////////////////////////////////////////////
		protected function update(event:Event):void
		{

			time_diff = getTimer() - last_time;
			FrameRate.update(time_diff);		
			last_time = getTimer();	

							
			//update mouse joint of coll system
			if (coll_system)
				coll_system.update();

			//update the memory system
			MemWatcher.update();

		
		}
		
		

		

		///////////////////////////////////////////////////////////////////////////
		//
		//
		///////////////////////////////////////////////////////////////////////////
		public function getStage():Stage  { return game_stage; }
		public function getScreenX():int  { return screen_size.x; }
		public function getScreenY():int  { return screen_size.y; }
		public function getCollSystem():CollSystem { return coll_system; }
		public function getTimeDiff():int { return time_diff; }

		


		///////////////////////////////////////////////////////////////////////////
		//
		//
		///////////////////////////////////////////////////////////////////////////
		public function setGameState(state:int):void
		{			
			game_state_obj.shutdown();
			game_state_obj = null;
			game_state = state;
		}

			
	}
}

