

///////////////////////////////////////////////////////////////////////
//
// represents a ball in the game
//
////////////////////////////////////////////////////////////////////////


package 
{

	include "ImportFile.as"


	public class Ball extends GameObj
	{


		private var parent_mechanix:Mechanix;
		private var hps:int;
		private var check_speed:Boolean = false;


		public static const TYPE_BASIC:uint 	= 0;	
		public static const TYPE_POWER:uint 	= 1;		
		public static const TYPE_SPINNER:uint 	= 2;		
		public static const TYPE_SPEEDY:uint 	= 3;		
		public static const TYPE_QUEEN:uint 	= 4;		
		public static const TYPE_KILLER:uint 	= 5;		
		public static const TYPE_KILLER_FRAGMENT:uint 	= 6;		

		private var type:int;
		private var lev:int;
		
		private var fragment:Boolean = false;
		private var fragment_delay:int = 60;
		
		
		///////////////////////////////////////////////////////////////////////////
		//
		//
		///////////////////////////////////////////////////////////////////////////
		public function Ball(game:Game, pobj:Sprite, pos:Point, in_parent_mechanix:Mechanix, vel:Point, in_type:int, thesize:Point, in_lev:int)
		{

			//set type
			type = in_type;
			
			lev = in_lev;
			
			//set collision info
			var coll_cat:uint = CollInfo.COLL_CATEGORY_BALL;
			var coll_mask:uint = CollInfo.COLL_CATEGORY_BAR|CollInfo.COLL_CATEGORY_BALL|CollInfo.COLL_CATEGORY_BARRIER|CollInfo.COLL_CATEGORY_COLLECTOR|CollInfo.COLL_CATEGORY_FLOWER|CollInfo.COLL_CATEGORY_BARRIER_OUTER|CollInfo.COLL_CATEGORY_DOOMSDAY;
 
 			//set flags
 			var flags:uint = GameObj.FLAG_DISPLAYABLE | GameObj.FLAG_COLLIDABLE | GameObj.FLAG_PUSH_TO_BACK;

 
 			//determine size based on type
			var size:Point;
			if (thesize != null)
			{
				size = thesize;
			
				//put an end to the fragmenting
				//since only killers fragment
				if (type == TYPE_KILLER)
				{
					if (size.x < 15)
						type = TYPE_KILLER_FRAGMENT;
				}
			}
			else
			{
				if (type == TYPE_BASIC)
					size = new Point(20,20);
				else if (type == TYPE_POWER)
					size = new Point(30,30);
				else if (type == TYPE_SPINNER)
					size = new Point(25,25);
				else if (type == TYPE_SPEEDY)
					size = new Point(20,20);
				else if (type == TYPE_QUEEN)
					size = new Point(20,20);
				else //if (type == TYPE_KILLER)
					size = new Point(40,40);
			}
			
			//call the super constructor
			super(game, pobj, pos, size, flags,coll_cat, coll_mask);

						
			//set parent mechanix of this ball and increment number of balls for this mechanix
			//Q: why doesn't the mechanix increment/decrement it's own ball counter?
			//A: because he's a lazy bastard, but also, he doesn't know when the ball dies
			//   technically i could setup a listener for the event of a ball's death
			//   but this way uses less resources.  after all, the ball itself will
			//   always know when it dies, and it can easily hold a pointer to its parent mechanix
			parent_mechanix = in_parent_mechanix;
			if (parent_mechanix)
				parent_mechanix.incCurrBallsOut();


			//set hit points based on ball type
			hps = 1;

			if (type == TYPE_KILLER || type == TYPE_KILLER_FRAGMENT)
				GameSounds.playKiller();


			//set vel adjustment
			setVelAdj(vel);

	
			//give the spinning balls a little extra kick
			if (type == TYPE_SPINNER)
			{
				var the_spin:int = Random.getInt(150) + 100;
				if (Random.getInt(2) ==  0)
					the_spin *= -1;

				setSpin(the_spin);	
			}


			//force ball to have its speed checked right at the start
			checkSpeed();

		}


		///////////////////////////////////////////////////////////////////////////
		//
		//
		///////////////////////////////////////////////////////////////////////////
		public override function shutdown():void
		{
			if (parent_mechanix)
				parent_mechanix.decCurrBallsOut();
				
			super.shutdown();
		}


		////////////////////////////////////////////////////////////////////////
		//
		// 
		//
		////////////////////////////////////////////////////////////////////////
		public override function update(time_diff:int):void
		{
			super.update(time_diff);

			//delete any balls that are outside the playing screen
			//(this is just super safety code)
			if ((x < 0)||
			    (x > game.getScreenX()) ||
			    (y < 0) ||
			    (y > game.getScreenY()))
			{
				delete_me = true;
			}

			//should only contrain speed when there is a vel change
			if (check_speed)
			{
				constrainSpeed();	
				check_speed = false;
			}
			

			if (fragment_delay > 0)
				fragment_delay--;

		}

		////////////////////////////////////////////////////////////////////////
		//
		// keeps speed of ball within an upper bound
		//
		////////////////////////////////////////////////////////////////////////
		private function constrainSpeed():void
		{		
			var vel:Point = getVel();
			var speed_max:int;

			if (lev == 1)
				speed_max = 500;
			else if (lev == 2)
				speed_max = 650;
			else
				speed_max = 800;
			if (Vector.getMagnitude(vel) > speed_max)
			{
				vel = Vector.normalize(vel);	
				vel = Vector.scale(vel,speed_max);
				setVel(vel);
			}
		}



		////////////////////////////////////////////////////////////////////////
		//
		//
		//
		////////////////////////////////////////////////////////////////////////
		public override function draw():void
		{
			graphics.clear();
	

			if (type == TYPE_BASIC)
			{
				graphics.beginFill(0x800080);
				graphics.drawCircle(0,0,size.x);
				graphics.endFill();			

				graphics.beginFill(0xf000f0);
				graphics.drawCircle(size.x/2,0,size.x * 0.5);
				graphics.endFill();			
			}
			else if (type == TYPE_KILLER || type == TYPE_KILLER_FRAGMENT)
			{
				graphics.beginFill(0xff0000);
				graphics.drawCircle(-size.x/2,size.y/2,size.x);
				graphics.endFill();			

				graphics.beginFill(0xff0000);
				graphics.drawCircle(size.x/2,size.y/2,size.x);
				graphics.endFill();			

				graphics.beginFill(0xffffff);
				graphics.drawCircle(0,0,size.x);
				graphics.endFill();			


				graphics.beginFill(0xff0000);
				graphics.drawCircle(0,-size.y/2,size.x);
				graphics.endFill();			

			}
			else if (type == TYPE_POWER)
			{
				graphics.beginFill(0x000080);
				graphics.drawCircle(0,0,size.x);
				graphics.endFill();			
				graphics.beginFill(0xe0e0e0);
				graphics.drawRoundRect(-size.x/2 * 1.35,-size.y/2 * 1.35,size.x*1.35,size.y*1.35,size.x*0.5,size.y*0.5);
				graphics.endFill();			
			}
			else if (type == TYPE_SPINNER)
			{
				graphics.beginFill(0x00a0a0);
				graphics.drawCircle(size.x/2,0,size.x * 0.9);
				graphics.endFill();			
				graphics.beginFill(0x0000ff);
				graphics.drawCircle(-size.x/2,0,size.x * 0.9);
				graphics.endFill();			
			}
			else if (type == TYPE_SPEEDY)
			{
				graphics.lineStyle(8, 0x808000, 1.0, false);
				graphics.beginFill(0x404000);
				graphics.drawCircle(size.x/2,0,size.x);
				graphics.endFill();			
				graphics.beginFill(0xffff00);
				graphics.drawCircle(size.x/2,0,size.x*0.8);
				graphics.endFill();			
			}
			else //if (type == TYPE_QUEEN)
			{
				graphics.beginFill(0x0000ff);
				graphics.drawCircle(0,0,size.x * 1.0);
				graphics.endFill();			

				graphics.beginFill(0xffffff);
				graphics.drawCircle(0,0,size.x * 0.75);
				graphics.endFill();			


				graphics.lineStyle(6, 0x000090, 1.0, false);
				graphics.moveTo(-size.x,0);
				graphics.lineTo(size.x,0);
				graphics.lineStyle(6, 0x000050, 1.0, false);
				graphics.moveTo(0,-size.y/2);
				graphics.lineTo(0,size.y/2);
				graphics.lineStyle(6, 0x000050, 1.0, false);
				graphics.moveTo(-size.x,-size.y/2);
				graphics.lineTo(size.x,size.y/2);
			}
			
						
		}


		///////////////////////////////////////////////////////////////////////////
		//
		//
		///////////////////////////////////////////////////////////////////////////
		public function fragmentMe():void
		{
			if (type == TYPE_KILLER && fragment_delay < 1)
			{
				fragment = true;
			}
		}


		///////////////////////////////////////////////////////////////////////////
		//
		//
		///////////////////////////////////////////////////////////////////////////
		public function getHps():int { return hps; }
		public function clearHps():void { hps = 0; }
		public function checkSpeed():void { check_speed = true; }
		public function shouldFragment():Boolean { return fragment; }
		public function getParentMechanix():Mechanix { return parent_mechanix; }
		public function getType():int { return type; }
		public function clearFragment():void { fragment = false; }
		public function getLev():int { return lev; }

	}
}