


////////////////////////////////////////////////////////////////////////
//
//
//
////////////////////////////////////////////////////////////////////////


package
{


	include "ImportFile.as"


	public class Barrier extends GameObj
	{
		private var type:int = TYPE_REG;

		public static const TYPE_REG:int     	    = 0;			
		public static const TYPE_BOX_A:int    	    = 1;			
		public static const TYPE_BOX_B:int    	    = 2;			

		private var base_pos:Point;
				
		private static const FLASH_LEN:int = 3;

		private var flash:int = -1;
		private var flash_active:Boolean = false;

	
		////////////////////////////////////////////////////////////////////////
		//
		//
		//
		////////////////////////////////////////////////////////////////////////
		public function Barrier(game:Game, pobj:Sprite, pos:Point, size:Point, flags:uint, is_outer:Boolean, in_type:int)
		{
			//store type
			type = in_type;
			
			//create coll info
			//note: an outer barrier is one of the four outer walls
			var coll_cat:uint;
			if (is_outer)
				coll_cat = CollInfo.COLL_CATEGORY_BARRIER_OUTER;
			else
				coll_cat = CollInfo.COLL_CATEGORY_BARRIER;
			var coll_mask:uint = CollInfo.COLL_CATEGORY_BAR|CollInfo.COLL_CATEGORY_BALL;

			//call super constructor			
			super(game,pobj, pos,size,flags, coll_cat, coll_mask);

			
			//save base position (this is where the barrier is fixed to 
			//note: base pos is not used/needed for COLLIDABLE_SOLID
			if (flags&FLAG_COLLIDABLE)
				base_pos = new Point(x,y);			
			
		}

		///////////////////////////////////////////////////////////////////////////
		//
		//
		///////////////////////////////////////////////////////////////////////////
		public override function update(time_diff:int):void
		{

			super.update(time_diff);

			processEffects();
			

			//calc x and y based on orig position
			if (flags&FLAG_COLLIDABLE)
			{
				//hard set vels			
				var pos:b2Vec2 = new b2Vec2(0,0);			
				coll_body.SetLinearVelocity(pos);
				coll_body.SetAngularVelocity(0.0);

				coll_body.m_position.x = base_pos.x;
				coll_body.m_position.y = base_pos.y;
				coll_body.SynchronizeShapes();		
			}

		}



		///////////////////////////////////////////////////////////////////////////
		//
		//
		///////////////////////////////////////////////////////////////////////////
		private function processEffects():void
		{
			if (flash == FLASH_LEN && flash_active == false)
				drawFlash();
			else if (flash == 0 && flash_active == true)
				drawUnflash();

			if (flash > -1)
				flash--;
		}
		


		///////////////////////////////////////////////////////////////////////////
		//
		//
		///////////////////////////////////////////////////////////////////////////
		public override function draw():void
		{
		
			graphics.clear();
		
			if (type == TYPE_REG)
			{
				graphics.lineStyle(8, 0x400040, 1.0, false);
				graphics.moveTo(-size.x/2,0);
				graphics.lineTo(size.x/2,0);

				graphics.lineStyle(3, 0x800080, 1.0, false);
				graphics.moveTo(-size.x/2,0);
				graphics.lineTo(size.x/2,0);

				graphics.lineStyle(1, 0x0000ff, 1.0, false);

				graphics.moveTo(-size.x/2,0);
				graphics.lineTo(size.x/2,0);
			}
			else if (type == TYPE_BOX_A)
			{
				graphics.lineStyle(2, 0x400040, 1.0, false);

				graphics.beginFill(0x800080);
				graphics.drawRoundRect(-size.x/2*0.6,-size.y/2,size.x*0.6,size.y,size.x*0.2,size.y*.2);
				graphics.endFill();

				graphics.beginFill(0x0000ff);
				graphics.drawRoundRect(-size.x/2*0.2,-size.y/2,size.x*0.2,size.y,size.x*0.2,size.y*.2);
				graphics.endFill();
			}
			else //if (type == TYPE_BOX_B)
			{
				graphics.lineStyle(2, 0x400040, 1.0, false);

				graphics.beginFill(0x800080);
				graphics.drawRoundRect(-size.x/2,-size.y/2*0.6,size.x,size.y*0.6,size.x*0.2,size.y*.2);
				graphics.endFill();

				graphics.beginFill(0x0000ff);
				graphics.drawRoundRect(-size.x/2,-size.y/2*0.2,size.x,size.y*0.2,size.x*0.2,size.y*.2);
				graphics.endFill();
			}

		}
		
		
		///////////////////////////////////////////////////////////////////////////
		//
		//
		///////////////////////////////////////////////////////////////////////////
		public function drawFlash():void
		{
			graphics.clear();

			flash_active = true;

			if (type == TYPE_REG)
			{

				graphics.lineStyle(8, 0x400040, 1.0, false);
				graphics.moveTo(-size.x/2,0);
				graphics.lineTo(size.x/2,0);

				graphics.lineStyle(3, 0x800080, 1.0, false);
				graphics.moveTo(-size.x/2,0);
				graphics.lineTo(size.x/2,0);

				graphics.lineStyle(1, 0x0000ff, 1.0, false);

				graphics.moveTo(-size.x/2,0);
				graphics.lineTo(size.x/2,0);

				graphics.lineStyle(1, 0xffffff, 1.0, false);
				graphics.moveTo(-size.x/2,0);
				graphics.lineTo(size.x/2,0);
			}
			else if (type == TYPE_BOX_A)
			{
				graphics.lineStyle(2, 0x400040, 1.0, false);

				graphics.beginFill(0x800080);
				graphics.drawRoundRect(-size.x/2*0.6,-size.y/2,size.x*0.6,size.y,size.x*0.2,size.y*.2);
				graphics.endFill();

				graphics.beginFill(0x0000ff);
				graphics.drawRoundRect(-size.x/2*0.2,-size.y/2,size.x*0.2,size.y,size.x*0.2,size.y*.2);
				graphics.endFill();

				graphics.beginFill(0xffffff);
				graphics.drawRoundRect(-size.x/2*0.2,-size.y/2,size.x*0.2,size.y,size.x*0.2,size.y*.2);
				graphics.endFill();

			}
			else //if (type == TYPE_BOX_B)
			{
				graphics.lineStyle(2, 0x400040, 1.0, false);

				graphics.beginFill(0x800080);
				graphics.drawRoundRect(-size.x/2,-size.y/2*0.6,size.x,size.y*0.6,size.x*0.2,size.y*.2);
				graphics.endFill();

				graphics.beginFill(0x0000ff);
				graphics.drawRoundRect(-size.x/2,-size.y/2*0.2,size.x,size.y*0.2,size.x*0.2,size.y*.2);
				graphics.endFill();

			
				graphics.beginFill(0xffffff);
				graphics.drawRoundRect(-size.x/2,-size.y/2*0.2,size.x,size.y*0.2,size.x*0.2,size.y*.2);
				graphics.endFill();
			}

		}


		///////////////////////////////////////////////////////////////////////////
		//
		//
		///////////////////////////////////////////////////////////////////////////
		public function drawUnflash():void
		{
			graphics.clear();

			flash_active = false;


			if (type == TYPE_REG)
			{
				graphics.lineStyle(8, 0x400040, 1.0, false);
				graphics.moveTo(-size.x/2,0);
				graphics.lineTo(size.x/2,0);

				graphics.lineStyle(3, 0x800080, 1.0, false);
				graphics.moveTo(-size.x/2,0);
				graphics.lineTo(size.x/2,0);

				graphics.lineStyle(1, 0x0000ff, 1.0, false);

				graphics.moveTo(-size.x/2,0);
				graphics.lineTo(size.x/2,0);
			}
			else if (type == TYPE_BOX_A)
			{
				graphics.lineStyle(2, 0x400040, 1.0, false);

				graphics.beginFill(0x800080);
				graphics.drawRoundRect(-size.x/2*0.6,-size.y/2,size.x*0.6,size.y,size.x*0.2,size.y*.2);
				graphics.endFill();

				graphics.beginFill(0x0000ff);
				graphics.drawRoundRect(-size.x/2*0.2,-size.y/2,size.x*0.2,size.y,size.x*0.2,size.y*.2);
				graphics.endFill();

			}
			else //if (type == TYPE_BOX_B)
			{
				graphics.lineStyle(2, 0x400040, 1.0, false);

				graphics.beginFill(0x800080);
				graphics.drawRoundRect(-size.x/2,-size.y/2*0.6,size.x,size.y*0.6,size.x*0.2,size.y*.2);
				graphics.endFill();

				graphics.beginFill(0x0000ff);
				graphics.drawRoundRect(-size.x/2,-size.y/2*0.2,size.x,size.y*0.2,size.x*0.2,size.y*.2);
				graphics.endFill();
			}

		}


		///////////////////////////////////////////////////////////////////////////
		//
		//
		///////////////////////////////////////////////////////////////////////////
		public function setFlash():void { flash = FLASH_LEN; }
	
	}
}