

///////////////////////////////////////////////////////////////////////
//
//
//
////////////////////////////////////////////////////////////////////////


package 
{

	include "ImportFile.as"


	public class Doomsday extends GameObj
	{
		private var hps:int = 1000;

		private var life_count_base:Number = 0;
		private var death_time:Number;	
		
		private var base_pos:Point;
						
		private var rot:Number = 0.0;

		private var doomsday_inner:DoomsdayInner;
		
		private var sound_played:Boolean = false;
		
		private var game_state:GameStatePlay;
	
		///////////////////////////////////////////////////////////////////////////
		//
		//
		///////////////////////////////////////////////////////////////////////////
		public function Doomsday(game:Game, pobj:Sprite, pos:Point, in_size:Point, in_death_time:int, in_game_state:GameStatePlay)
		{
		
			game_state = in_game_state;
			
			//set collision info
			var coll_cat:uint = CollInfo.COLL_CATEGORY_DOOMSDAY;
			var coll_mask:uint = CollInfo.COLL_CATEGORY_FLOWER|CollInfo.COLL_CATEGORY_BALL;

			//call the super constructor
			super(game, pobj, pos,in_size,(GameObj.FLAG_DISPLAYABLE|GameObj.FLAG_COLLIDABLE),coll_cat, coll_mask);

			//set base pos		
			base_pos = new Point(pos.x,pos.y);

			//set death time and change markers
			death_time = in_death_time;	
			
			//setup doomsday inner
			var size_inner:Point = new Point(this.size.x * 0.75, this.size.y * 0.75);
			doomsday_inner = new DoomsdayInner(game,this,new Point(0,0), size_inner);
			
			//set doomsday to initial state
			reset();
			
			
			doomsday_inner.doubleClickEnabled = true;
			doomsday_inner.addEventListener(MouseEvent.DOUBLE_CLICK, changeBarDirection);

		}
		


	

		////////////////////////////////////////////////////////////////////////
		//
		// 
		//
		////////////////////////////////////////////////////////////////////////
		public override function update(time_diff:int):void
		{

			//call super update		
			super.update(time_diff);

			//calc time diff since last time instruction pointer was here
			var clock:Date = new Date();
			var curr_time:Number = clock.time;					
			var diff:Number = (curr_time - life_count_base)/1000;

			//if counter has not reached the time of death, the make sure device stays in place at the top
			//adjust rotation (based on how close to death)
			var rotdiff:Number = (1.0 - ((death_time - diff)/death_time)) * 0.5;
			rot += rotdiff; 
			if(diff < death_time)
			{
				setPosRot(base_pos.x,base_pos.y,rot);
			}
			else
			{
				if (sound_played == false)
				{	
					GameSounds.playDanger();
					sound_played = true;
				}

				setRot(rot);
			}

			//activate glow if its about to fall
			if ((death_time - diff) < (death_time * 0.1))
				doomsday_inner.activateGlow();

		}


		///////////////////////////////////////////////////////////////////////////
		//
		//
		///////////////////////////////////////////////////////////////////////////
		public override function shutdown():void
		{
			doomsday_inner.removeEventListener(MouseEvent.DOUBLE_CLICK, changeBarDirection);

			doomsday_inner.shutdown();

			super.shutdown();
		}


		////////////////////////////////////////////////////////////////////////
		//
		//
		//
		////////////////////////////////////////////////////////////////////////
		public override function draw():void
		{
			var dsize:int;
			dsize = size.x;

			var mat:Matrix = new Matrix();
			mat.createGradientBox(size.x/2,size.y/2,1.57,0,0);	

			var c:Array = new Array();
			var a:Array = new Array();
			var r:Array = new Array();

			a = [1,1];
			r = [0,255];

			if (game_state.getBar().getRotAmt() > 0.0)
				c = [0xdd0010,0x200010];
			else
				c = [0x1000dd,0x100020];
			
			graphics.beginGradientFill(GradientType.LINEAR,
							c,
							a,
							r,
							mat,
							SpreadMethod.PAD,
							InterpolationMethod.RGB,
							0.0);


			graphics.drawCircle(0,0,dsize);
			graphics.endFill();			
		}



		////////////////////////////////////////////////////////////////////////
		//
		//
		//
		////////////////////////////////////////////////////////////////////////
		public function reset():void
		{
			var clock:Date = new Date();
			var curr_time:Number = clock.time;	
		
			life_count_base = curr_time;
			
			doomsday_inner.decactivateGlow();

			draw();
			doomsday_inner.draw();

			sound_played = false;

		}



		///////////////////////////////////////////////////////////////////////////
		//
		//
		///////////////////////////////////////////////////////////////////////////
		public function changeBarDirection(event:Event):void
		{
			game_state.getBar().changeDirection();
			draw();
		}


		///////////////////////////////////////////////////////////////////////////
		//
		//
		///////////////////////////////////////////////////////////////////////////
		public function getHps():int { return hps; }
		public function clearHps():void { hps = 0; }

	}
}