

////////////////////////////////////////////////////////////////////////
//
// flower
//
////////////////////////////////////////////////////////////////////////


package
{

	include "ImportFile.as"


	public class Flower extends GameObj
	{
		private static const BASE_AGE:uint = 3;		
		private static const  MAX_AGE:uint = 10;	
		private var age:Number = BASE_AGE;

		private var budding_count:int = 0;
		private static const BUD_INTERVAL:uint = 3;	

		private var base_size:Point;
		private var base_pos:Point;
		
		private var game_state:GameState;
				
		public static const AGE_STAGE_1:int = 2;
		public static const AGE_STAGE_2:int = 4;
		public static const AGE_STAGE_3:int = 7;

		public var flower_info:FlowerInfo;

		private var use_coll_mesh:Boolean;

		
		////////////////////////////////////////////////////////////////////////
		//
		//
		//
		////////////////////////////////////////////////////////////////////////
		public function Flower(game:Game, pobj:Sprite, pos:Point, in_size:Point, in_game_state:GameState, in_flower_info:FlowerInfo, stem_type:Boolean, in_use_coll_mesh:Boolean)
		{


			//if passing in flower info, then build flower according to that, otherwise make random new one	
			if (in_flower_info == null)
				flower_info = new FlowerInfo();
			else
				flower_info = in_flower_info;

			//store game state
			game_state = in_game_state;


			//store base info
			base_pos = new Point(pos.x,pos.y);
			base_size = new Point(in_size.x,in_size.y);	


			//make stem
			new Stem(game,this,0,0,base_size,GameObj.FLAG_DISPLAYABLE,stem_type);

			//call super const
			//Q: why is there no COLLIDABLE FLAG being set here?
			//A: the collision stucture for a flower must encompass the stem, and all the petals,
			//   so insead of making seperate (more expensive) collision areas for each subpart,
			//   a simple coll mesh is made to represent the whole flower - see further down
			super(game, pobj, pos,in_size, GameObj.FLAG_DISPLAYABLE,0,0);
			
			//make petals		
			var petal:Petal;
			var i:int;
			var j:int;
			for (i=2;i>=0;i--)
			{				
				var curr_degree:int = flower_info.array_degree_offset[i];
				for(j=0;j<flower_info.array_num_petals[i];j++)
				{
					var pet:Petal = new Petal(game,this,0,0,flower_info.array_petal_shape[i],base_size,GameObj.FLAG_DISPLAYABLE,flower_info.array_color_set[i],i);
					pet.setRot(curr_degree);
					curr_degree += flower_info.array_degree_inc[i];
				}
			}

			use_coll_mesh = in_use_coll_mesh;

			//adjust flower size based on age
			adjustFlower();

		}





		////////////////////////////////////////////////////////////////////////
		//
		//
		//
		////////////////////////////////////////////////////////////////////////
		public override function shutdown():void
		{
			if (use_coll_mesh)
				game.getCollSystem().destroyCollisionMesh(coll_body);

			super.shutdown();
		}
		
		

							
		////////////////////////////////////////////////////////////////////////
		//
		//
		//
		////////////////////////////////////////////////////////////////////////
		public function grow(amt:int):void
		{
			if (amt <= 0)
				return; 

			budding_count++;

			//if budding count has reached the interval - shoot a bud
			//into the air, and reset the budding count
			if (budding_count >= BUD_INTERVAL)
			{
				addBud();
				budding_count = 0;
			}


			//increase flower age
			age++;
			if (age > MAX_AGE)
				age = MAX_AGE;

			adjustFlower();

		}
		
		////////////////////////////////////////////////////////////////////////
		//
		//
		//
		////////////////////////////////////////////////////////////////////////
		public function growFull():void
		{

			age = MAX_AGE;
			adjustFlower();

		}
		

		////////////////////////////////////////////////////////////////////////
		//
		//
		//
		////////////////////////////////////////////////////////////////////////
		public function setAge(in_age:int):void
		{

			age = in_age;
			
			if (age < 1)
				age = 1;
			if (age > MAX_AGE)
				age = MAX_AGE;

			adjustFlower();

		}

		
 		///////////////////////////////////////////////////////////////////////////
		//
		//
		///////////////////////////////////////////////////////////////////////////
		private function addBud():void
		{

			var bud:Bud = new Bud(game, game, new Point(this.x,this.y - 30), game_state);
			((GameStatePlay)(game_state)).gameObjListAdd(bud);

			GameSounds.playBud();

		}



		////////////////////////////////////////////////////////////////////////
		//
		//
		//
		////////////////////////////////////////////////////////////////////////
		public function shrink(amt:int):void
		{
				
			//decrease flowers age		
			age -= amt;
			if (age < 0)
				age = 0;

			adjustFlower();

		}




		////////////////////////////////////////////////////////////////////////
		//
		//
		//
		////////////////////////////////////////////////////////////////////////
		private function adjustFlower():void
		{
			//calc size based on base size and age
			var new_size:Point = new Point();
			new_size.x = base_size.x + ((base_size.x) * age);
			new_size.y = base_size.y + ((base_size.y) * age);
				
			//resize the center of the flower
			size.x = new_size.x;
			size.y = new_size.y;

			//resize each of the children
			var gobj:GameObj;
			var i:int;
			for(i = 0; i < this.numChildren;i++)
			{
				gobj = (GameObj)(this.getChildAt(i));
				gobj.setSizeX(new_size.x);
				gobj.setSizeY(new_size.y);
			}

			//redraw the center of the flower
			draw();

			//redraw each of the children			
			for(i = 0; i < this.numChildren;i++)
			{
				gobj = (GameObj)(this.getChildAt(i));
				gobj.draw();
			}
			
			
			if (use_coll_mesh)
			{
				//destroy and recreate the collision object
				if (coll_body)
					game.getCollSystem().destroyCollisionMesh(coll_body);


				//create a collision mesh - attach it to the flower (represented in objects as the
				//center bud - the reason we are doing this separately is because
				//the flower center bud obj is only a small circle while the actual flower
				//needs to be represented by a big circle and a stem
				flags = GameObj.FLAG_DISPLAYABLE|GameObj.FLAG_COLLIDABLE_SOLID;
				collision_category = CollInfo.COLL_CATEGORY_FLOWER;
				collision_mask = CollInfo.COLL_CATEGORY_BALL|CollInfo.COLL_CATEGORY_BAR|CollInfo.COLL_CATEGORY_DOOMSDAY;
				game.getCollSystem().complexCreateStart(new Point(x,y), collision_category, collision_mask);
				var stem_size:Point = new Point(size.x/6,size.y * 2);
				game.getCollSystem().complexCreateAddBox(new Point(0,size.y*2.5), stem_size);
				game.getCollSystem().complexCreateAddCircle(new Point(0,0), size);
				coll_body = game.getCollSystem().complexCreateEnd();
			}

		}					


		////////////////////////////////////////////////////////////////////////
		//
		//
		//
		////////////////////////////////////////////////////////////////////////
		public function isFullyGrown():Boolean
		{
			
			if (age >= MAX_AGE)
				return true;
				
			return false;
		}

		////////////////////////////////////////////////////////////////////////
		//
		//
		//
		////////////////////////////////////////////////////////////////////////
		public function isDead():Boolean
		{
	
			if (age <= 0)
				return true;
				
			return false;
		}


		////////////////////////////////////////////////////////////////////////
		//
		//
		//
		////////////////////////////////////////////////////////////////////////
		public override function draw():void
		{		
			//draw with center at center!
			var grfx:Graphics = this.graphics;

			grfx.clear();


			grfx.beginGradientFill(GradientType.RADIAL,
					       flower_info.colors1,
					       flower_info.alphas1,
					       flower_info.ratios1,
					       null,
					       SpreadMethod.PAD,
					       InterpolationMethod.RGB,
					       0.0);


			//adjust y based on flower's age
			y = base_pos.y - (15 * age);


			grfx.drawEllipse(-size.x/2,-size.y/2,size.x * 0.9,size.y * 0.9);
			graphics.endFill();


		}


		////////////////////////////////////////////////////////////////////////
		//
		//
		//
		////////////////////////////////////////////////////////////////////////
		public function getAge():int { return age; }

	}
}