

////////////////////////////////////////////////////////////////////////
//
// base class for shooter enemy
//
////////////////////////////////////////////////////////////////////////


package 
{

	include "ImportFile.as"


	public class Mechanix extends GameObj
	{

		private var type:int;
		public static const TYPE_BASIC:uint 	= 0;	
		public static const TYPE_POWER:uint 	= 1;		
		public static const TYPE_SPINNER:uint 	= 2;		
		public static const TYPE_SPEEDY:uint 	= 3;		
		public static const TYPE_QUEEN:uint 	= 4;		

		private var mode:int = 0;
		private static const MODE_SELECT_POS:int = 0;
		private static const MODE_MOVE:int 	 = 1;
		private static const MODE_FIRE:int 	 = 2;
		
		private var next_action:int = 0; 
		private static const ACTION_WAIT:int 	   = 0;
		private static const ACTION_MOVE_RIGHT:int = 1;
		private static const ACTION_MOVE_LEFT:int  = 2;
		private static const ACTION_FIRE:int       = 3;

		private var move_speed:int = 1;
		private var MAX_MOVE_SPEED:int = 10;

		private static const MARKER_MAX:int = 500.0;
		private static const MARKER_MAX_10:int = MARKER_MAX * 0.10;
		private static const MARKER_MAX_25:int = MARKER_MAX * 0.25;
		private static const MARKER_MAX_50:int = MARKER_MAX * 0.50;
		private static const MARKER_MAX_75:int = MARKER_MAX * 0.75;
		private static const MARKER_MAX_90:int = MARKER_MAX * 0.90;
		private var marker_next:int = 0;
		private var marker_curr:Number;

		private var max_balls_out:int = 1; 
		private var curr_balls_out:int = 0; 
			
		private var game_state:GameStatePlay;
			
		private var ability:int = 1;

		
		////////////////////////////////////////////////////////////////////////
		//
		//
		//
		////////////////////////////////////////////////////////////////////////
		public function Mechanix(game:Game, pobj:Sprite, pos:Point, in_size:Point, in_type:uint, in_ability:int, in_game_state:GameStatePlay)
		{
			
			//set info based on type		
			type = in_type;


			//store game state
			game_state = in_game_state;
		

			//store ability of this mechanix
			//the higher the number, the more difficult this mechanix is for the player
			//hehe, the input field for ability is called inability
			ability = in_ability;
		

			//set coll info
			var coll_cat:uint = CollInfo.COLL_CATEGORY_MECHANIX;
			var coll_mask:uint = CollInfo.COLL_CATEGORY_BALL|CollInfo.COLL_CATEGORY_BAR;


			//set flags
			var flags:uint = GameObj.FLAG_DISPLAYABLE|GameObj.FLAG_COLLIDABLE_SOLID;


			//call super consctructor
			super(game, pobj, pos, in_size, flags, coll_cat, coll_mask);
						

			//calc a current starting point
			marker_curr = (Number)(Random.getInt(MARKER_MAX));

	
			//move speed
			if (type == TYPE_SPEEDY)
			{
				if (ability == 1)
					move_speed = 3;
				else if (ability == 2)
					move_speed = 5;
				else if (ability == 3)
					move_speed = 10;
			}
			else
			{
				if (ability == 1)
					move_speed = 1;
				else if (ability == 2)
					move_speed = 2;
				else if (ability == 3)
					move_speed = 3;
			}

			//set mechanix type specific properties		
			if (type == TYPE_BASIC)
			{
				max_balls_out = 1;
			}
			else if (type == TYPE_POWER)
			{
				max_balls_out = 1;
			}
			else if (type == TYPE_SPINNER)
			{
				max_balls_out = 1;	
			}
			else if (type == TYPE_SPEEDY)
			{
				max_balls_out = 1;
			}
			else //if (type == TYPE_QUEEN)
			{
				max_balls_out = 2;
			}

			if (move_speed > MAX_MOVE_SPEED)
				move_speed = MAX_MOVE_SPEED;
		
			
		}
		


		///////////////////////////////////////////////////////////////////////////
		//
		// mechanix just move a long a track.
		//     it chooses a number between 0 and MARKER_MAX which then
		//     gets translated into an x,y pos on the track
		//
		///////////////////////////////////////////////////////////////////////////
		public override function update(time_diff:int):void
		{
			var marker_last:int = marker_curr;
		
			//decide what mechanix should do next
			think();


			//do the next action
			if (next_action == ACTION_MOVE_RIGHT)
				marker_curr += move_speed;
			else if (next_action == ACTION_MOVE_LEFT)
				marker_curr -= move_speed;
			

			//only update mechanix's pos info if marker has changed
			if (marker_curr != marker_last)
			{
				if (marker_curr < 0)
					marker_curr = 0;
				if (marker_curr >= MARKER_MAX)
					marker_curr = MARKER_MAX;
			
				//calc x and y based on marker system
				var yn:Number = 0.0;
				var xn:Number = 0.0;

				if (marker_curr <= MARKER_MAX_25)
				{
					x = 8;

					yn = 1.0 - (marker_curr/(MARKER_MAX_25));
					y = yn * ((game.getScreenY()) * 0.50);
					if (y < 8)
						y = 8;

				}
				else if (marker_curr <= MARKER_MAX_75)
				{
					y = 8;

					xn = ((MARKER_MAX_75) - marker_curr);
					xn = 1.0 - (xn/(MARKER_MAX_50));
					x = xn * (game.getScreenX());

					if (x < 8)
						x = 8;
					else if (x > game.getScreenX() - 8)
						x = game.getScreenX() - 8;
				}
				else // > 75%
				{
					x = game.getScreenX() - 8;

					yn = MARKER_MAX - marker_curr;
					yn = 1.0 - (yn/(MARKER_MAX_25));
					y = yn * ((game.getScreenY()) * 0.50);

					if (y < 8)
						y = 8;

				}

				//set body info based on new x/y			
				coll_body.m_position.x = x;
				coll_body.m_position.y = y;
				coll_body.m_rotation = 0.0;
				coll_body.SynchronizeShapes();
			}
		}



		///////////////////////////////////////////////////////////////////////////
		//
		// forces the mechanix to shoot if it can shoot right now
		//
		///////////////////////////////////////////////////////////////////////////
		public function forceToShoot():void
		{
			//skip out if its ball is already in play
			if (curr_balls_out >= max_balls_out)
				return;

			//force it to shoot if its not already shooting
			if (mode != MODE_FIRE)
			{
				mode = MODE_FIRE;
				next_action = ACTION_WAIT;					
			}
		}
		
		
		///////////////////////////////////////////////////////////////////////////
		//
		//
		///////////////////////////////////////////////////////////////////////////
		private function think():void
		{
			
			if (mode == MODE_SELECT_POS)
			{
				marker_next = Random.getInt(MARKER_MAX);
				mode = MODE_MOVE;
				next_action = ACTION_WAIT;

			}
			else if (mode == MODE_MOVE)
			{
				if (Math.abs(marker_curr - marker_next) <= MAX_MOVE_SPEED)  
				{
					mode = MODE_FIRE;
					next_action = ACTION_WAIT;					
				}

				if (marker_curr < marker_next)
					next_action = ACTION_MOVE_RIGHT;
				else if (marker_curr > marker_next)
					next_action = ACTION_MOVE_LEFT;

			}
			else if (mode == MODE_FIRE)
			{
				if (curr_balls_out < max_balls_out)
					createBall();				
			
				mode = MODE_SELECT_POS;

			}			
				
		}
		


		///////////////////////////////////////////////////////////////////////////
		//
		// Q: why is there no destroyBall function in this class?
		// A: the mechanix doesn't destroy the ball. the gamestateplay
		//    determines the time of the balls death (i.e. in collector,
		//    or out of bounds)
		//
		///////////////////////////////////////////////////////////////////////////
		private function createBall():void
		{
			var ball:Ball;
			var ball_type:int = type;
						
			//change mechanix ball to type to killer
			if (ability > 1)
			{
				if (Random.getInt(100) < 10)
					ball_type = Ball.TYPE_KILLER;
			}
	

			//set speed for ball
			if (ability == 1)
				base_vel = 175;
			else if (ability == 2)
				base_vel = 200;
			else //
				base_vel = 225;
				
			var spx:Number;
			var spy:Number;
			var base_vel:int;

			if (marker_curr <= MARKER_MAX_25)
			{
				spx = Random.getInt(base_vel) + base_vel;
			}
			else if (marker_curr >= MARKER_MAX_75)
			{
				spx = Random.getInt(-base_vel) - base_vel;
			}
			else //
			{
				spx = Random.getInt(base_vel*4) - (base_vel*2);
			}
			
			if (marker_curr <= MARKER_MAX_10)
			{
				spy = Random.getInt(-base_vel) - base_vel;
			}
			else if (marker_curr <= MARKER_MAX_25)
			{
				spy = Random.getInt(base_vel*4) - base_vel;
			}
			else if (marker_curr >= MARKER_MAX_90)
			{
				spy = Random.getInt(-base_vel) - base_vel;
			}
			else if (marker_curr >= MARKER_MAX_75)
			{
				spy = Random.getInt(base_vel*4) - (base_vel*2);
			}
			else //
			{
				spy = Random.getInt(base_vel) + base_vel;
			}


			ball = new Ball(game, game, new Point(x,y),this,new Point(spx,spy),ball_type,null,ability);


			//add ball to game obj list	
			game_state.gameObjListAdd(ball);
									


		}
		
		
		///////////////////////////////////////////////////////////////////////////
		//
		//
		///////////////////////////////////////////////////////////////////////////
		public function decCurrBallsOut():void 	{ curr_balls_out--;}
		public function incCurrBallsOut():void 	{ curr_balls_out++;}
		

		////////////////////////////////////////////////////////////////////////
		//
		//
		//
		////////////////////////////////////////////////////////////////////////
		public override function draw():void
		{
			graphics.clear();

			graphics.lineStyle(5, 0xffffff, 1.0, false);

			if (this.type == TYPE_BASIC)
			{

				graphics.beginFill(0x800080);
				graphics.drawCircle(0,0,size.x);
				graphics.endFill();			

				graphics.beginFill(0xf000f0);
				graphics.drawCircle(size.x/2,0,size.x * 0.5);
				graphics.endFill();			
			}
			else if (this.type == TYPE_POWER)
			{

				graphics.beginFill(0x000080);
				graphics.drawCircle(0,0,size.x);
				graphics.endFill();			
				graphics.beginFill(0xe0e0e0);
				graphics.drawRoundRect(-size.x/2 * 1.35,-size.y/2 * 1.35,size.x*1.35,size.y*1.35,size.x*0.5,size.y*0.5);
				graphics.endFill();			
			}
			else if (this.type == TYPE_SPINNER)
			{
				graphics.beginFill(0x00a0a0);
				graphics.drawCircle(size.x/2,0,size.x * 0.9);
				graphics.endFill();			
				graphics.beginFill(0x0000ff);
				graphics.drawCircle(-size.x/2,0,size.x * 0.9);
				graphics.endFill();			

			}
			else if (this.type == TYPE_SPEEDY)
			{
				graphics.beginFill(0xffff00);
				graphics.drawCircle(size.x/2,0,size.x);
				graphics.endFill();			
			}
			else //if (this.type == TYPE_QUEEN)
			{
				graphics.beginFill(0x0000ff);
				graphics.drawCircle(0,0,size.x * 1.0);
				graphics.endFill();			

				graphics.beginFill(0xffffff);
				graphics.drawCircle(0,0,size.x * 0.75);
				graphics.endFill();			


				graphics.lineStyle(6, 0x000090, 1.0, false);
				graphics.moveTo(-size.x,0);
				graphics.lineTo(size.x,0);
				graphics.lineStyle(6, 0x000050, 1.0, false);
				graphics.moveTo(0,-size.y/2);
				graphics.lineTo(0,size.y/2);
				graphics.lineStyle(6, 0x000050, 1.0, false);
				graphics.moveTo(-size.x,-size.y/2);
				graphics.lineTo(size.x,size.y/2);

			}

		}

	
	}
}